/***************************************************/
/* program to extract data from the NAVSTAR XR4-PC */
/* provided by BANCOMM DIV. of DATUM INC.                               */
/* Version 1.0  20 Feb 1992  (dennis 408 578-4161) */
/* compiled with TURBO C                                                                        */
/***************************************************/
/* Version 1.1  31 Mar 1992  (dennis again)                     */
/*  Added variable (msh) milliseconds hundreds          */ 
/*  loaded by check_toa. If msh is greater than 4  */
/*  then main() waits for a new time packet until  */
/*  msh is =< than 4. This trap guarantess that    */
/*  the time loaded to the bc620 will be incre-    */
/*  mented by the bc620 firmware. The increment         */
/*  routine is called shortly after mid-second of       */
/*  the 1PPS epoch. The time in the time packet    */
/*  always refers to the time at the preceding     */
/*  1PPS rising edge. The time is no longer incre- */
/*  in the function check_toa().                                                   */
/***************************************************/ 

/***************************************************
/* Version bcXR4B.c 28 April 1992 (dennis)                      */
/*       This version was created to operate as a fore- */
/*  ground task to discipline an external oscil-        */
/*       lator. The primary role of the foreground task */
/*       is to monitor packet $POLYF which provides the */
/*       fix mode status of the XR4PC.  When the fix            */
/*       mode is !(2 || 3) then the bc620 is instructed */
/*       to cease the disciplining function and hold            */
/*  the D/A output at the last computed value.          */
/***************************************************/
/* Version 1.2 30 Apr 1993 (Greg Dowd)             */
/*  Modified for bc620at rev D boards.  Added      */
/*  assert to tell 620 to act on FIFO data.  Also  */
/*  added code to pull base address and local      */
/*  offset from the command line.                  */
/***************************************************/

#include <stdio.h>
#include <dos.h>

#define ACK 0x0B
#define Assert outp(base+ACK,0x80)
#define Wait while((!(inp(base+ACK) & 0x1)) && (!kbhit()))

char getdata(void) ;
void getime(void) ;

char    achar,header[6],major[7],day[3],month[3],year[5] ;
char    msh ;
char    done = 0 ;
int     i,years ;
int     normal[12] = {0,31,59,90,120,151,181,212,243,273,304,334} ;
int     leap[12]   = {0,31,60,91,121,152,182,213,244,283,305,335} ;
int     *cum ;
char    pathByteA = 0 ;  
char    pathByteB = 0 ;
int     base = 0x340 ;
char    local_offset = -8 ;

main(int argc, char *argv[])
{
   int bh, bt, bu ;

   if ( ( argc > 1) && ( strlen ( argv[1] ) <=3 ) )
    {
	bh = *argv[1];
	if ( ( bt = *( argv[1] + 1 ) ) > '9' )
	    bt += 9;
	if ( ( bu = *( argv[1] + 2 ) ) > '9' )
	    bu += 9;
	base = ( 0x100 * ( bh & 0x0F ) ) + ( 0x10 * ( bt & 0x0F ) ) + ( bu & 0x0F );
    }
    else
	base = get_port() ;

    if ( argc > 2 )
	local_offset = atoi ( argv[2] ) ;

   clrscr() ;

   /* wait until xr4pc is tracking 3 or more sats */

	printf("waiting for tracking status\n") ;
	while((!done)&&(!kbhit())) {
		if(getdata() == '$') {
			for(i=0 ; i<5 ; i++) header[i] = getdata() ;
		if(!strcmp(header,"POLYF")) {
				achar = getdata() ;
				achar = getdata() ;
				if((achar=='2')||(achar=='3')) done = 1 ;
				}
			}
		}
	printf("tracking status = %c\n",achar) ;


  /* now select bc620 mode 2 (ext 1PPS mode) */

	outportb(base+15,1) ; /* select page 1 */
	outportb(base+14,1) ; /* SOH */
	outportb(base+14,'A') ;
	outportb(base+14,0x32) ;
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
	printf("mode set = 2\n") ;

   /* select 1 MHz output rate */

	outportb(base+15,1) ; /* select page 1 */
   achar = inportb(base+0) ;
   outportb(base+0,achar|0x80) ;
	printf("1 MHz TTL output selected\n") ;

	/* set pathByteA = 4 (inhibit jamsync check) */

	outportb(base+15,1) ;
	outportb(base+14,1) ;
	outportb(base+14,'P') ;
	outportb(base+14,pathByteA | 4) ;
	outportb(base+14,pathByteB ) ;
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
	printf("Auto jamsynch inhibited\n") ;

	/* set jamsync on */

	outportb(base+15,1) ;
	outportb(base+14,1) ;
	outportb(base+14,'C') ;
	outportb(base+14,'3') ;
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
	printf("Jamsync forced\n") ;

	/* clear the reference 1PPS ACK bit */
	outportb(base+15,1) ;
	outportb(base+11,2) ;
   /* wait for bit to be set */
	while(!(inportb(base+11)&2)) ;
	outportb(base+11,2) ;
   /* wait for bit to be set */
	while(!(inportb(base+11)&2)) ;
	outportb(base+11,2) ;
   /* wait for bit to be set */
	while(!(inportb(base+11)&2)) ;


	/* set pathByteA = 0 (enable jamsync check) */
	outportb(base+15,1) ;
	outportb(base+14,1) ;
	outportb(base+14,'P') ;
	outportb(base+14,pathByteA & ~4 ) ;
	outportb(base+14,pathByteB ) ;
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
	printf("Auto jamsync enabled\n") ;

   /* set D/A to 2048 */
   outportb(base+15,1) ;
	outportb(base+14,1) ;
	outportb(base+14,'D') ;
	outportb(base+14,8) ;
   outportb(base+14,0) ;
   outportb(base+14,0) ;
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
	printf("D/A value output\n") ;

	done = 0 ;
  
   while(!done) {
      clrscr() ;
		if(getdata() == '$') {
			for(i=0 ; i<5 ; i++) header[i] = getdata() ;
		if(!strcmp(header,"GPZDA")) {
				check_toa() ;
			if(msh <= 4) {
				getime() ;
					printf("synch time  : ") ;
					putchar(major[0]) ;
					putchar(major[1]) ;
					putchar(':') ;
					putchar(major[2]) ;
					putchar(major[3]) ;
					putchar(':') ;
					putchar(major[4]) ;
					putchar(major[5]) ;
				printf("\nsynch month : %s\n",month) ;
				printf("synch day   : %s\n",day) ;
				printf("synch year  : %s\n",year) ;
				set_time() ;
					done = 1 ;
					}
				}
			}
		}

	while(!kbhit()) {
		if(getdata() == '$') {
			for(i=0 ; i<5 ; i++) header[i] = getdata() ;
		if(!strcmp(header,"GPZDA")) {
			getime() ;
				gotoxy(1,7) ;
				printf("UTC time: ") ;
				putchar(major[0]) ;
				putchar(major[1]) ;
				putchar(':') ;
				putchar(major[2]) ;
				putchar(major[3]) ;
				putchar(':') ;
				putchar(major[4]) ;
				putchar(major[5]) ;
		printf("\nmonth: %s  ",month) ;
			printf("day: %s  ",day) ;
			printf("year: %s\n",year) ;
			}
		if(!strcmp(header,"POLYF")) {
				achar = getdata() ;
				achar = getdata() ;
				gotoxy(1,10) ;                  
			printf("tracking status : ") ;
			putchar(achar) ;
				if((achar=='2')||(achar=='3')) {
					printf("\ndiscipline enabled ") ;
				   if(pathByteA & 8) {
						outportb(base+15,1) ;
						outportb(base+14,1) ;
						outportb(base+14,'P') ;
						outportb(base+14,0) ;
						outportb( base+14,pathByteB ) ;
						outportb(base+14,0x17) ;
						Assert ;
						Wait ;
					pathByteA = 0 ;
					}
				}
			else {
				printf("\ndiscipline disabled") ;
					if( ! ( pathByteA & 8 ) ) {
						outportb(base+15,1) ;
						outportb(base+14,1) ;
						outportb(base+14,'P') ;
						outportb(base+14,8) ;
						outportb( base+14,pathByteB ) ;
						outportb(base+14,0x17) ;
						Assert ;
						Wait ;
					pathByteA = 8 ;
					}
				}
			}
		}
		}      
	gotoxy(1,14) ;
	printf("bcxr4B exit\n\n") ; 
}


check_toa() {
	outportb(base+15,0) ;           /* page 0 */
	achar = inportb(base) ;   /* req */
	msh = (inportb(base+6) & 0xF0)>>4 ;
	printf("synch minor : ") ;
	putchar(msh + 0x30) ;
	putchar((inportb(base+7) & 0x0F) + 0x30) ;
	putchar(((inportb(base+7) & 0xF0)>>4) + 0x30) ;
	putchar((inportb(base+7) & 0x0F) + 0x30) ;
	putchar(((inportb(base+8) & 0xF0)>>4) + 0x30) ;
	putchar((inportb(base+8) & 0x0F) + 0x30) ;
	putchar(0x0A) ;
	putchar(0x0D) ;
}       
	

char getdata(void) {
		while(!(inportb(0x301)&1)) ;
      return(inportb(0x300)) ;
}


void getime(void) {
	achar = getdata() ;
	major[0] = getdata() ;  /* hrs ten */
	major[1] = getdata() ;  /* hrs unit */
	major[2] = getdata() ;  /* min ten */
	major[3] = getdata() ;  /* min unit */
	major[4] = getdata() ;  /* sec ten */
	major[5] = getdata() ;  /* sec unit */
	achar = getdata() ;             /* discard ',' */
   day[0] = getdata() ;    /* day ten */
   day[1] = getdata() ;    /* day unit */
   achar = getdata() ;     /* discard ',' */
	month[0] = getdata() ;  /* month ten */
	month[1] = getdata() ;  /* month unit */
	achar = getdata() ;     /* discard ',' */
	year[0] = getdata() ;   /* year thou */
	year[1] = getdata() ;   /* year hun */
	year[2] = getdata() ;   /* year ten */
	year[3] = getdata() ;   /* year unit */

	years = (year[0]&15) * 1000 +
			  (year[1]&15) * 100 +
			  (year[2]&15) * 10 +
			  (year[3]&15) ;
}


set_time() {
int days,hours ;
 
	if(years%4 == 0) cum = leap ;
	else cum = normal ;
	
	i = (month[0]&15)*10 + (month[1]&15) ;
	
	days = *(cum+i-1) + (day[0]&15)*10 + (day[1]&15) ;              
	
	hours = (major[0]&15)*10 + (major[1]&15) + local_offset ;
	if(hours<0) {
		hours += 24 ;
		days-- ;
		}
	if(hours>23) {
		hours -= 24 ;
		days++ ;
		}

	major[0] = (hours/10) | 0x30 ;
	major[1] = (hours%10) | 0x30 ;
	
	outportb(base+15,1) ;   /* select page 1 */
	outportb(base+14,1) ;   /* header byte */
	outportb(base+14,'B') ; /* ID byte */
	outportb(base+14,major[5]) ;
	outportb(base+14,major[4]) ;
	outportb(base+14,major[3]) ;
	outportb(base+14,major[2]) ;
	outportb(base+14,major[1]) ;
	outportb(base+14,major[0]) ;
	
	outportb(base+14,(days%10)|0x30) ;
	outportb(base+14,((days%100)/10)|0x30) ;
	outportb(base+14,(days/100)|0x30) ;
	
	outportb(base+14,0x17) ;
	Assert ;
	Wait ;
}       

int get_port()
{
     /* Function returns base address of selected port */

     char key[2];

     do
     {
	  clrscr ();
	  gotoxy(22,1);
	  printf("BANCOMM bc620AT DEMONSTRATION PROGRAM");
	  gotoxy(1,5);
	  puts("                  0) 0200H - 020FH          16) 0300H - 030FH");
	  puts("                  1) 0210H - 021FH          17) 0310H - 031FH");
	  puts("                  2) 0220H - 022FH          18) 0320H - 032FH");
	  puts("                  3) 0230H - 023FH          19) 0330H - 033FH");
	  puts("                  4) 0240H - 024FH          20) 0340H - 034FH");
	  puts("                  5) 0250H - 025FH          21) 0350H - 035FH");
	  puts("                  6) 0260H - 026FH          22) 0360H - 036FH");
	  puts("                  7) 0270H - 027FH          23) 0370H - 037FH");
	  puts("                  8) 0280H - 028FH          24) 0380H - 038FH");
	  puts("                  9) 0290H - 029FH          25) 0390H - 039FH");
	  puts("                 10) 02A0H - 02AFH          26) 03A0H - 03AFH");
	  puts("                 11) 02B0H - 02BFH          27) 03B0H - 03BFH");
	  puts("                 12) 02C0H - 02CFH          28) 03C0H - 03CFH");
	  puts("                 13) 02D0H - 02DFH          29) 03D0H - 03DFH");
	  puts("                 14) 02E0H - 02EFH          30) 03E0H - 03EFH");
	  puts("                 15) 02F0H - 02FFH          31) 03F0H - 03FFH");
	  printf("\n\n                      Enter I/O Port Base Address (0-31): ");

	  gets(key);           /* Get port base address */
     }
     while ((atoi(key) < 0) || (atoi(key) > 31));   /* loop not valid entry */

     return ( 0x200 + 16 * atoi(key));
}

void Wait_awhile( void )
{
     while((!(inportb(base+11) & 1))&&!kbhit()) ; /* Wait for FIFO rx bit set */
     printf("acknowledge bit = %d\n",(int)inport(base+11)&1) ;
     printf("hit any key to continue") ;
     achar = getche() ;
     return ;
}

